#!/usr/bin/wish
# Author: Travis Veldkamp
# Date: December 2002

# A simple TCL library for handling LCD configurations in files

#TODO: add XPM as available file type


# Configuration...
set LCD_XSIZE 5
set LCD_YSIZE 8

namespace eval ::LCDGrid {
   global LCD_XSIZE LCD_YSIZE 

   variable CellState

   ###############
   # BEGIN PROCS #
   ###############

   ################################################################################
   #                          lcdgrid  Initialize                                 #
   # Set all entries in the grid to off                                           #
   ################################################################################
   proc ::LCDGrid::Initialize {} {
      global LCD_YSIZE LCD_XSIZE

      Clear
   }


   ################################################################################
   #                          lcdgrid  GetByte                                    #
   # Gets a row from the grid and returns in a byte (each pixel as a bit)         #
   # RowNum is between 1 and LCD_YSIZE                                            #
   ################################################################################
   proc ::LCDGrid::GetByte { RowNum } {

      set CurByte 0x00
      if {$RowNum <= [GetHeight]} {
         for {set xidx 0} {$xidx < $LCD_XSIZE} {incr xidx} {
            set place [expr ($LCD_XSIZE - 1) - $xidx]
            set CurByte [expr $CurByte | ([GetCellState $xidx $RowNum] << $place)]
         }
      }

      return $CurByte
   }


   ################################################################################
   #                          lcdgrid  Save                                       #
   # Save the state of the grid to a file for later use                           #
   ################################################################################
   proc ::LCDGrid::Save { FileName } {
      global LCD_YSIZE LCD_XSIZE
      variable StatusText

      Config::Destroy

      Config::AddVariable "Size" "X" $LCD_XSIZE
      Config::AddVariable "Size" "Y" $LCD_YSIZE

      for {set yidx 0} {$yidx < $LCD_YSIZE} {incr yidx} {
         # Build up the byte for this row...
         set CurByte 0x00
         for {set xidx 0} {$xidx < $LCD_XSIZE} {incr xidx} {
            set place [expr ($LCD_XSIZE - 1) - $xidx]
            set CurByte [expr $CurByte | ([GetCellState $xidx $yidx] << $place)]
         }
         set octalbyte [format "%o" $CurByte]
         Config::AddVariable "Bytes" "Byte$yidx" $CurByte
      }

      if {[Config::Write $FileName] == 1} {
         set StatusText "Saved to $FileName"
         set retval 1
      } else {
         set StatusText "ERROR: failed to save $FileName"
         set retval 0
      }

      return $retval
   }

   ################################################################################
   # Clear - clear the grid                                                       #
   # Set all entries in the grid to off                                           #
   ################################################################################
   proc ::LCDGrid::Clear {{Fill 0}} {
      global LCD_YSIZE LCD_XSIZE

      for {set yidx 0} {$yidx < $LCD_YSIZE} {incr yidx} {
         for {set xidx 0} {$xidx < $LCD_XSIZE} {incr xidx} {
            SetCellState $xidx $yidx $Fill
         }
      }

   }

   ################################################################################
   #                          lcdgrid  Load                                      #
   ################################################################################
   proc ::LCDGrid::Load { FileName } {
      global LCD_YSIZE LCD_XSIZE

      Config::Destroy

      Config::Read $FileName

      set xmax [Config::GetValue "Size" "X"]
      set ymax [Config::GetValue "Size" "Y"]
      puts "Load: xmax: $xmax, ymax:$ymax"


      if {(($xmax != $LCD_XSIZE) || ($ymax != $LCD_YSIZE))} {

         set retval 0
      } else {
         # Size is as expected...
         set retval 1

         for {set yidx 0} {$yidx < $LCD_YSIZE} {incr yidx} {
            set CurByte [Config::GetValue "Bytes" "Byte$yidx"]
            if {[string length $CurByte] != 0} {
               # Value was set, now decode the bits...
               for {set xidx 0} {$xidx < $LCD_XSIZE} {incr xidx} {
                  set place [expr ($LCD_XSIZE - 1) - $xidx]
                  SetCellState $xidx $yidx [expr ($CurByte & (1 << $place)) > 0]
               }
            } else {
               # Value was not set in config file... error
               set retval 0
            }
         }

      }

      return $retval

   }

   ################################################################################
   # Gets the state of the specified cell                                         #
   ################################################################################
   proc ::LCDGrid::GetCellState {x y} {
      variable CellState

      return $CellState($x,$y)
   }

   ################################################################################
   # Sets the state of the specified cell                                         #
   ################################################################################
   proc ::LCDGrid::SetCellState {x y Val} {
      variable CellState

      set CellState($x,$y) $Val
   }

   ################################################################################
   # Given a square number, toggles the state                                     #
   ################################################################################
   proc ::LCDGrid::ToggleCell {x y} {
      variable CellState

      set CellState($x,$y) [expr ! $CellState($x,$y)]

   }

   ################################################################################
   # Return the width of the grid                                                 #
   ################################################################################
   proc ::LCDGrid::GetWidth { } {
      global LCD_XSIZE

      return $LCD_XSIZE
   }

   ################################################################################
   # Return the height of the grid                                                #
   ################################################################################
   proc ::LCDGrid::GetHeight { } {
      global LCD_YSIZE

      return $LCD_YSIZE
   }

   ####################
   # E N D  P R O C S #
   ####################

   Initialize

} ;# end of LCDGrid namespace
   
