#!/usr/bin/tcl
# Programmed by: Travis Veldkamp (tlvmail@bigfoot.com)
# December 2002
# Version 1.0

#TODO: add "M" for Character Mapping... not sure what it does, actually

# A simple TCL library for manipulating an LCD attached to a computer and driven using the LCDmod driver
# Available routines are:
#
#    - Open - attempt to open a channel to the LCD
#       - Parameters: DeviceFile (ie /dev/lcd)
#       - returns: 0 on failure, 1 on success
#
#    - Close - close the channel to the LCD
#       - Parameters: none
#       - Returns: nothing
#
#    - Connected - check if a channel to the LCD has been successfully opened (via Open)
#       - Parameters: none
#       - Returns: 1 if Open has been called successfully, else 0
#      
#    - Clear - clears the LCD display, if connected
#       - Parameters: none
#       - Returns: nothing
#
#    - SendString - sends an ASCII string to the LCD, if connected
#       - Parameters: ASCII_String
#       - Returns: nothing
#
#    - CursorHome - sends cursor to 1,1 position, if connected
#       - Parameters: none
#       - Returns: nothing
#
#    - CursorDown - sends cursor down 1 row, if connected
#       - Parameters: none
#       - Returns: nothing
#
#    - CursorUp - sends cursor up 1 row, if connected
#       - Parameters: none
#       - Returns: nothing
#
#    - CursorRight - sends cursor 1 column to the right, if connected
#       - Parameters: none
#       - Returns: nothing
#
#    - CursorLeft - sends cursor 1 column to the left, if connected
#       - Parameters: none
#       - Returns: nothing
#
#    - MoveCursor - sends cursor to Xpos, YPos if connected
#       - Parameters: Xpos, Ypos
#       - Returns: nothing
#
#    - ToggleBacklight - if LCD is equipped with a backlight, toggles the state
#       - Parameters: none
#       - Returns: nothing
#
#    - EraseEOL - erase from the cursor position to the end of the line (doesn't move cursor)
#       - Parameters: none
#       - Returns: nothing
#
#    - PgmCustom - saves a new character to CGRAM
#       - Parameters: CGRAM_Index Byte1 Byte2 Byte3 Byte4 Byte5 Byte6 Byte7 Byte8
#       - Returns: nothing
#
#    - SendCustom - draws a character from CGRAM to the LCD
#       - Parameters: CGRAM_Index 
#       - Returns: nothing
#
#    - SetLinewrap - sets linewrap mode
#       - Parameters: State (1 for on, 0 for off)
#       - Returns: nothing
#
# Low level routines (likely not used by most users) are:
#    - GetFileID - after calling Open successfully, this will be set to the channel in use
#       - Parameters: none
#       - Returns: fileid if channel open, else empty string
#
#    - SendEscSeq - sends the escape sequence to the LCD, if connected
#       - Parameters: none
#       - Returns: nothing
#
namespace eval ::LCD {
   variable fileid ""

   ################################
   # Get the name of the LCD      #
   # connection if exists else    #
   # empty                        #
   ################################
   proc ::LCD::GetFileID {} {
      variable fileid

      set retval ""
      if [info exists fileid] {
         set retval $fileid
      }
      return $retval
   }

   ################################
   # Check if a connection to the #
   # LCD has been established     #
   ################################
   proc ::LCD::Connected {} {

      set retval 0
      if {[string length [GetFileID]] != 0} {
         set retval 1
      }

      return $retval
   }

   ################################
   # Send the escape sequence to  #
   # the LCD if connected         #
   ################################
   proc ::LCD::SendEscSeq {} {
      if [Connected] {
         # escape sequence is ESC and open bracket
         # 0x1B = ESC
         puts -nonewline [GetFileID] [binary format H2 1b]
         puts -nonewline [GetFileID] "\["
         flush [GetFileID]
      }
   }

   ################################
   # Send a string to the LCD if  #
   # connected                    #
   ################################
   proc ::LCD::SendString {SendBuf} {

      if [Connected] {
         puts -nonewline [GetFileID] $SendBuf
         flush [GetFileID]
      }
   }

   ################################
   # Clear the LCD                #
   ################################
   proc ::LCD::Clear {} {

      #Send ESC and open bracket then J to clear screen
      SendEscSeq
      SendString "J"
   }

   ################################
   # Send the cursor home         #
   ################################
   proc ::LCD::CursorHome {} {
      #Send ESC and open bracket then H to send cursor back to 1,1
      SendEscSeq
      SendString "H"
   }

   ################################
   # Send the cursor up a row     #
   ################################
   proc ::LCD::CursorUp {} {
      #Send ESC and open bracket then A to send cursor up a row
      SendEscSeq
      SendString "A"
   }

   ################################
   # Send the cursor down a row   #
   ################################
   proc ::LCD::CursorDown {} {
      #Send ESC and open bracket then B to send cursor down a row
      SendEscSeq
      SendString "B"
   }

   ################################
   # Send the cursor right a col  #
   ################################
   proc ::LCD::CursorRight {} {
      #Send ESC and open bracket then C to send cursor right a column
      SendEscSeq
      SendString "C"
   }

   ################################
   # Send the cursor left a col   #
   ################################
   proc ::LCD::CursorLeft {} {
      #Send ESC and open bracket then D to send cursor left a column
      SendEscSeq
      SendString "D"
   }

   ################################
   # Toggle the backlight         #
   ################################
   proc ::LCD::ToggleBacklight {} {
      #Send ESC and open bracket then b to toggle the backlight
      SendEscSeq
      SendString "b"
   }

   ################################
   # Erase to the end of the line #
   # and don't move the cursor    #
   ################################
   proc ::LCD::EraseEOL {} {
      #Send ESC and open bracket then K to clear to end of line
      SendEscSeq
      SendString "K"
   }

   ################################
   # Generate a new character in  #
   # the CGRAM                    #
   ################################
   proc ::LCD::PgmCustom {idx byte0 byte1 byte2 byte3 \
                             byte4 byte5 byte6 byte7 } {
      set retval 0
      if {($idx >= 0) && ($idx < 8)} {
         #Send ESC and open bracket then K to clear to end of line
         SendEscSeq
         SendString "R"
         SendString $idx
         SendString [binary format H2 [format "%0.2X" $byte0]]
         SendString [binary format H2 [format "%0.2X" $byte1]]
         SendString [binary format H2 [format "%0.2X" $byte2]]
         SendString [binary format H2 [format "%0.2X" $byte3]]
         SendString [binary format H2 [format "%0.2X" $byte4]]
         SendString [binary format H2 [format "%0.2X" $byte5]]
         SendString [binary format H2 [format "%0.2X" $byte6]]
         SendString [binary format H2 [format "%0.2X" $byte7]]
         set retval 1
      }
      return $retval
   }

   ################################
   # Move cursor to X,Y           #
   ################################
   proc ::LCD::MoveCursor {X Y} {
      #Send ESC and open bracket then Y to set cursor move mode
      #Then, send coordinates
      #Coordinates always have 0x1F added
      SendEscSeq
      SendString "Y"
      #Send X coordinate
      SendString [binary format H2 [format "%0.2X" [expr $X + 0x1F]]]
      #Send Y coordinate
      SendString [binary format H2 [format "%0.2X" [expr $Y + 0x1F]]]
   }

   ################################
   # Set Linewrap on/off - use 1  #
   # for on, 0 for off            #
   ################################
   proc ::LCD::SetLinewrap {Value} {
      #Send ESC and open bracket then V/W to turn linewrap on/off
      SendEscSeq
      if {$Value} {
         # Turn linewrap on
         SendString "V"
      } else {
         # Turn linewrap off
         SendString "W"
      }
   }

   ################################
   # Send "idx"th character in    #
   # CGRAM                        #
   ################################
   proc ::LCD::SendCustom {idx} {
      #Access CGRAM characters with 0 - 7

      set retval 0

      if {($idx >= 0) && ($idx < 8)} {
         SendString [binary format H2 [format "%0.2X" $idx]]
         set retval 1
      }
    
      return $retval
   }


   ################################
   # Open a connection to the LCD #
   ################################
   proc ::LCD::Open {LCDDev} {
      variable fileid

      if [catch {set fileid [open $LCDDev w]}] {
        set retval 0

         #Make sure fileid is still empty...
         set fileid ""
      } else {
         fconfigure $fileid -translation binary -encoding binary
         set retval 1
      }
      return $retval
   }

   ################################
   # Close the LCD                #
   ################################
   proc ::LCD::Close {} {
      variable fileid

      if [Connected] {
         close $fileid
         set fileid ""
      }
   }

} ; #end LCD namespace
